/* --------------------------------------------------------------
 gulp_general_composer.js 2019-03-19
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

'use strict';

/**
 * Gulp General Composer Task
 *
 * Execute this task after a "composer install" in order to prepare the php vendor files
 * of the shop (src/vendor).
 *
 * The original problem is that composer will install many unnecessary files that sometimes
 * have incompatible licenses. This tasks will move only the required files at the src/vendor directory.
 *
 * @param {Gulp} gulp Gulp Instance
 * @param {Object} $ Contains the automatically loaded gulp plugins.
 *
 * @return {Function} Returns the gulp task definition.
 */
module.exports = function(gulp, $) {
	const fs = require('fs');
	const del = require('del');
	const exec = require('child_process').exec;
	const through = require('through2');
	
	function transpile() {
		return through.obj(function(chunk, enc, cb) {
			exec('composer_components/bin/php7to5 convert composer_components/gambio-hub/hubpublic src/vendor/gambio-hub/hubpublic', (error,
				stdout, stderr) => {
				if (error) {
					$.util.log($.util.colors.red(`Transpile PHP sources error: ${error}`));
					return;
				}
				
				$.util.log(`stdout: ${stdout}`);
				
				del.sync('../vendor/gambio-hub/hubpublic/docs');
				del.sync('../vendor/gambio-hub/hubpublic/tests');
			});
			
			cb(null, chunk);
		});
	}
	
	function miscellaneous() {
		return through.obj(function(chunk, enc, cb) {
			const placeholder = '<?php // Composer Autoloader Workaround';
			// The debug bar component now introduces the symfony folder already.
			if (!fs.existsSync('../vendor/symfony')) {
				fs.mkdirSync('../vendor/symfony');
			}
			fs.mkdirSync('../vendor/symfony/polyfill-mbstring');
			fs.writeFileSync('../vendor/symfony/polyfill-mbstring/bootstrap.php', placeholder);
			
			fs.mkdirSync('../vendor/symfony/polyfill-ctype');
			fs.writeFileSync('../vendor/symfony/polyfill-ctype/bootstrap.php', placeholder);
			
			fs.mkdirSync('../vendor/symfony/polyfill-php73');
			fs.writeFileSync('../vendor/symfony/polyfill-php73/bootstrap.php', placeholder);
			
			fs.mkdirSync('../vendor/paragonie');
			fs.mkdirSync('../vendor/paragonie/random_compat');
			fs.mkdirSync('../vendor/paragonie/random_compat/lib');
			fs.writeFileSync('../vendor/paragonie/random_compat/lib/random.php', placeholder);
			
			fs.mkdirSync('../vendor/illuminate');
			fs.mkdirSync('../vendor/illuminate/support');
			fs.writeFileSync('../vendor/illuminate/support/helpers.php', placeholder);
			
			fs.mkdirSync('../vendor/myclabs');
			fs.mkdirSync('../vendor/myclabs/deep-copy');
			fs.mkdirSync('../vendor/myclabs/deep-copy/src');
			fs.mkdirSync('../vendor/myclabs/deep-copy/src/DeepCopy');
			fs.writeFileSync('../vendor/myclabs/deep-copy/src/DeepCopy/deep_copy.php', placeholder);
			
			cb(null, chunk);
		});
	}
	
	function task() {
		del.sync('../vendor');
		
		// Move the Hub connector files into the shop directory.
		del.sync([
			'../gambio_hub_callback.php',
			'../gambio_installer/sql/zz_gambio_hub.sql',
			'../GXModules/Gambio/Hub',
			'../includes/modules/payment/gambio_hub.php',
			'../version_info/gambio_hub-*.php',
		]);
		
		gulp.src([
			'composer_components/gambio-hub/hub-connector/variants/GX 4.3.1.0 - 4.3x/**/*',
			'!**/*/gambio_hub_callback.php'
		]).pipe(gulp.dest('src'));
		
		gulp.src([
			'composer_components/gambio-hub/hub-connector/variants/GX 4.3.1.0 - 4.3x/GXModules/Gambio/Hub/gambio_hub_callback.php',
		]).pipe(gulp.dest('src'))
			// .pipe(transpile())
			.pipe(miscellaneous());
		
		// Move auto updater files into shop directory.
		del.sync([
			'../admin/html/assets/javascript/modules/auto_updater',
			'../GXMainComponents/Controllers/HttpView/Admin/auto_updater',
			'../GXMainComponents/Extensions/AutoUpdater',
			'../lang/english/original_sections/admin/auto_updater',
			'../lang/german/original_sections/admin/auto_updater',
			'../lang/german/original_sections/admin/auto_updater',
			'../version_info/auto_updater-*.php',
			'../gambio_store.php',
		]);
		gulp.src([
			'composer_components/gambio/auto-updater/src/**/*',
			'!composer_components/gambio/auto-updater/src/gambio_updater/**/*',
		]).pipe(gulp.dest('src'));
		
		// Move the responsive file manager to the src directory.
		gulp.src(['composer_components/gambio/responsive-filemanager/**/*'])
			.pipe(gulp.dest('../ResponsiveFilemanager'));
		
		// Move the required packages to the src/vendor directory and remove the unnecessary files.
		gulp.src('composer_components/autoload.php')
			.pipe(gulp.dest('../vendor'));
		
		gulp.src('composer_components/composer/*')
			.pipe(gulp.dest('../vendor/composer'));
		
		gulp.src(['composer_components/gambio/codeigniter-db/**/*'])
			.pipe(gulp.dest('../vendor/gambio/codeigniter-db'));
		
		gulp.src('composer_components/phpmailer/phpmailer/*.php')
			.pipe(gulp.dest('../vendor/phpmailer/phpmailer'));
		
		gulp.src('composer_components/natxet/cssmin/src/*.php')
			.pipe(gulp.dest('../vendor/natxet/cssmin/src/'));
		
		gulp.src('composer_components/firebase/php-jwt/src/*.php')
			.pipe(gulp.dest('../vendor/firebase/php-jwt/src/'));
		
		gulp.src('composer_components/slim/slim/Slim/**/*')
			.pipe(gulp.dest('../vendor/slim/slim/Slim/'));
		
		gulp.src(['composer_components/gambio/gprotector/**/*'])
			.pipe(gulp.dest('../GProtector'));

		gulp.src(['composer_components/gambio/pdf-creator/**/*'])
			.pipe(gulp.dest('../PdfCreator'));
		
		gulp.src('composer_components/mpdf/mpdf/**/*')
			.pipe($.ignore.exclude(function(file) {
				const ttfonts = file.path.indexOf('ttfonts/') > -1
					&& file.path.indexOf('DejaVuSerifCondensed.ttf') === -1;
				
				const qrcode = file.path.indexOf('qrcode') > -1;
				
				return ttfonts || qrcode;
			}))
			.pipe(gulp.dest('../vendor/mpdf/mpdf/'));
		
		gulp.src('composer_components/setasign/fpdi/**/*')
			.pipe(gulp.dest('../vendor/setasign/fpdi/'));
		
		gulp.src('composer_components/true/punycode/src/**/*')
			.pipe(gulp.dest('../vendor/true/punycode/src'));
		
		gulp.src('composer_components/phpseclib/phpseclib/phpseclib/**/*')
			.pipe(gulp.dest('../vendor/phpseclib/phpseclib/phpseclib'));
		
		gulp.src('composer_components/davefx/phplot/phplot/phplot.php')
			.pipe(gulp.dest('../vendor/davefx/phplot/phplot'));
		
		gulp.src('composer_components/smarty/smarty/libs/**/*')
			.pipe(gulp.dest('../vendor/smarty/smarty/libs'));
		
		gulp.src('composer_components/maximebf/debugbar/src/**/*')
			.pipe(gulp.dest('../vendor/maximebf/debugbar/src'));
		
		gulp.src('composer_components/pclzip/pclzip/pclzip.lib.php')
			.pipe(gulp.dest('../vendor/pclzip/pclzip/'));
		
		gulp.src('composer_components/symfony/var-dumper/**/*')
			.pipe($.ignore.exclude(function(file) {
				const testFolders = file.path.indexOf('/Test') !== -1 || file.path.indexOf('/Tests') !== -1;
				
				// Holds the .gitignore, changelog, composer.json, license, readme and more.
				const utilityFiles = file.path.indexOf('.gitignore') !== -1 ||
					file.path.indexOf('CHANGELOG.md') !== -1 ||
					file.path.indexOf('composer.json') !== -1 ||
					file.path.indexOf('LICENSE') !== -1 ||
					file.path.indexOf('phpunit.xml.dist') !== -1 ||
					file.path.indexOf('README.md') !== -1;
				
				return testFolders || utilityFiles;
			}))
			.pipe(gulp.dest('../vendor/symfony/var-dumper/'));
		
		gulp.src('composer_components/psr/log/Psr/**/*')
			.pipe(gulp.dest('../vendor/psr/log/Psr'));
		
		gulp.src('composer_components/psr/http-message/src/**/*')
			.pipe(gulp.dest('../vendor/psr/http-message/src'));
		
		gulp.src('composer_components/guzzlehttp/guzzle/src/**/*')
			.pipe(gulp.dest('../vendor/guzzlehttp/guzzle/src'));
		
		gulp.src('composer_components/guzzlehttp/promises/src/**/*')
			.pipe(gulp.dest('../vendor/guzzlehttp/promises/src'));
		
		gulp.src('composer_components/guzzlehttp/psr7/src/**/*')
			.pipe(gulp.dest('../vendor/guzzlehttp/psr7/src'));
		
		gulp.src('composer_components/ralouphie/getallheaders/src/**/*')
			.pipe(gulp.dest('../vendor/ralouphie/getallheaders/src'));
		
		gulp.src('composer_components/monolog/monolog/src/**/*')
			.pipe(gulp.dest('../vendor/monolog/monolog/src'));
		
		gulp.src('composer_components/sentry/sentry/**/*')
			.pipe(gulp.dest('../vendor/sentry/sentry'));
		
		gulp.src('composer_components/mtdowling/cron-expression/**/*')
			.pipe(gulp.dest('../vendor/mtdowling/cron-expression'));
		
		gulp.src('composer_components/league/flysystem/**/*')
			.pipe(gulp.dest('../vendor/league/flysystem'));
		
		gulp.src('composer_components/league/flysystem-sftp/**/*')
			.pipe(gulp.dest('../vendor/league/flysystem-sftp'));
		
		gulp.src('composer_components/cache/filesystem-adapter/**/*')
			.pipe(gulp.dest('../vendor/cache/filesystem-adapter'));
		
		gulp.src('composer_components/nikic/fast-route/**/*')
			.pipe(gulp.dest('../vendor/nikic/fast-route'));
		
		gulp.src('composer_components/pimple/pimple/**/*')
			.pipe(gulp.dest('../vendor/pimple/pimple'));
		
		gulp.src('composer_components/php-curl-class/php-curl-class/**/*')
			.pipe(gulp.dest('../vendor/php-curl-class/php-curl-class'));
		
		gulp.src('composer_components/container-interop/container-interop/**/*')
			.pipe(gulp.dest('../vendor/container-interop/container-interop'));
		
		gulp.src('composer_components/psr/container/**/*')
			.pipe(gulp.dest('../vendor/psr/container'));
		
		gulp.src('composer_components/doctrine/**/*')
			.pipe(gulp.dest('../vendor/doctrine'));
		
		gulp.src('composer_components/league/**/*')
			.pipe(gulp.dest('../vendor/league'));
		
		gulp.src('composer_components/phpmailer/phpmailer/**/*')
			.pipe(gulp.dest('../vendor/phpmailer/phpmailer'));
        
        if(fs.existsSync('composer_components/gambio-developer/gambio-cloud/src/')) {
            gulp.src(['composer_components/gambio-developer/gambio-cloud/src/**/*'])
                .pipe(gulp.dest('src'));
        }
        
        if (fs.existsSync('composer_components/gambio-developer/demoshop-1/src/')) {
            gulp.src('composer_components/gambio-developer/demoshop-1/src/**/*')
                .pipe(gulp.dest('src'));
        }
        
        if (fs.existsSync('composer_components/gambio-developer/demoshop-2/src/')) {
            gulp.src('composer_components/gambio-developer/demoshop-2/src/**/*')
                .pipe(gulp.dest('src'));
        }
        
        if (fs.existsSync('composer_components/gambio-developer/demoshop-3/src/')) {
            gulp.src('composer_components/gambio-developer/demoshop-3/src/**/*')
                .pipe(gulp.dest('src'));
        }
	}
	
	task.__description = 'Will copy the composer files to the src/vendor directory (they need to be '
		+ 'installed first with composer install).';
	
	return task;
};
